#include <TimerOne.h>
#include <SD.h>
#include <Wire.h>
#include <stdio.h>
#include <avr/wdt.h>

#define    STOP  0
#define    CW    1
#define    CCW   2

int     DS1307_ADDRESS=0x68;
int     val = 0;
File    fl;					// ファイル変数
int     adata[16];			// アナログデータ用バッファ
char    strg[32],fname[32];	// 文字処理用
byte    r_sec,r_min,r_hour,r_week,r_day,r_mon,r_year;
short   hsec;				// 分・秒を秒でカウントする
volatile    byte    tflag;	// 0.35秒、割り込みで使う

void    setup() {
    digitalWrite(A2,HIGH);
    digitalWrite(10,LOW);
    pinMode(9,INPUT);		// スイッチ入力
    pinMode(2,OUTPUT);		// motor CW
    pinMode(3,OUTPUT);		// motor CCW
    pinMode(4,OUTPUT);		// アナログセレクト0
    pinMode(5,OUTPUT);		// アナログセレクト1
    pinMode(6,OUTPUT);		// アナログセレクト2
    pinMode(7,INPUT);		// 停電検出
    pinMode(9,INPUT);		// スイッチ入力
    pinMode(10,OUTPUT);		// LED
    pinMode(A2,OUTPUT);		// counter
    motor(STOP);
    digitalWrite(9,HIGH);	// スイッチ入力プルアップ
    Wire.begin();
    SD.begin(8);			// microSD CS : pin8
    digitalWrite(10,LOW);	// SD.beginでHIGHになってしまう
    Serial.begin(9600);
    Timer1.initialize(350000); // Timer1 : 0.35秒
    Timer1.attachInterrupt( timerIsr ); // timerIsrをアタッチ
    tflag = 0;
    while(tflag==0);
    wdt_enable(WDTO_2S);
}

// Timer1 割り込み 0.35秒カウント
void timerIsr() {
    tflag=1;
}

#define    DATANUM    100

void    loop() {
    int    i,j,k,l;

    while(digitalRead(9)==HIGH) wdt_reset();	// スイッチ入力待ち
    delay(10);
    digitalWrite(10,HIGH);
    tflag = 0;
    readTime();
    sprintf(fname,"%c%c%c%c%03X.D00",H2C(r_year),H2C(r_mon),H2C(r_day),H2C(r_hour),hsec);
    Serial.println(fname);
    if((fl = SD.open(fname,FILE_WRITE))==false) for(;;);	// ファイルオープン失敗時は停止
    digitalWrite(10,LOW);
    for(i=0;i<DATANUM;i++) {
        readTime();
        sprintf(strg,"%02X/%02X/%02X %02X:%02d:%02d",r_year,r_mon,r_day,r_hour,r_min,r_sec);
        Serial.println(strg);
        fl.println(strg);			// 日付タイトル書き込み
        motor(CW);					// CW 35秒 with 100個分データ取り込み&書き込み
        while(tflag==0);
        for(k=0;k<100;k++) {		// 往路100データをサンプル
            tflag = 0;
            digitalWrite(10,HIGH);
            data_read();
            for(l=0;l<15;l++) {		// 16データをカンマ区切りで書き込み
                fl.print(adata[l],DEC);
                fl.print(",");
            }
            fl.println(adata[15],DEC);
//            fl.flush();			// 即クローズできるようにフラッシュしておく
            digitalWrite(10,LOW);
            while(tflag==0) {		// 0.35秒待ち
                if(digitalRead(7)==LOW) {	//　停電?
                    fl.close();		// とりあえず即ファイルクローズ
                    for(;;);		// 永久ループ
                }
                if(digitalRead(9)==HIGH) goto lll;	// スイッチ:停止?
            }
            wdt_reset();
        }
        fl.flush();
        motor(CCW);					// CCW 35秒
        for(k=0;k<100;k++) {		// 復路はサンプルしない
            digitalWrite(10,HIGH);
            if(k==99) digitalWrite(A2,LOW);
            tflag = 0;
            delay(5);
            digitalWrite(10,LOW);
            while(tflag==0) {
                if(digitalRead(7)==LOW) {	// 停電?
                    fl.close();		// とりあえず即ファイルクローズ
                    for(;;);		// 永久ループ
                }
                if(digitalRead(9)==HIGH) goto lll;	// スイッチ:停止?
            }
            if(k==99) digitalWrite(A2,HIGH);
            wdt_reset();
        }
    }
lll:
    fl.close();
    motor(STOP);
    while(digitalRead(9)==HIGH) wdt_reset();
    delay(10);
}

// 16chアナログ信号入力
void    data_read(){
    int    i;
    for(i=0;i<8;i++) {
        digitalWrite(4,bitRead(i,0));
        digitalWrite(5,bitRead(i,1));
        digitalWrite(6,bitRead(i,2));
        delayMicroseconds(10);
        adata[i]=analogRead(A0);
        adata[i+8]=analogRead(A1);
    }
}

// ACモーター制御
void    motor(byte cond) {
    if(cond==STOP) {digitalWrite(2,HIGH);digitalWrite(3,HIGH);}
    if(cond==CW)   {digitalWrite(3,HIGH);digitalWrite(2,LOW );}
    if(cond==CCW)  {digitalWrite(2,HIGH);digitalWrite(3,LOW );}
}

// 00h～31hの二桁HEX -> 0～Uの一文字へ変換
char    H2C(byte bt) {
    char    rc;
    
    rc = ((bt>>4)*10+(bt&0xf)) & 0x3f;	// 10進変換
    if(rc<=9) return(rc+'0'); else return((rc-10)+'A');
}

// RTCから日付時刻データ読み込み
void readTime() {
    byte rval;
    
    Wire.beginTransmission(DS1307_ADDRESS);
    Wire.write(val);
    Wire.endTransmission();
    Wire.requestFrom(DS1307_ADDRESS,7);
    rval=Wire.read();		// r_secは10進にしておく
    r_sec = ((rval>>4)*10+(rval&0xf));
    rval=Wire.read();		// r_minは10進にしておく
    r_min = ((rval>>4)*10+(rval&0xf));
    hsec = (r_min*60+r_sec) & 0xfff;
    r_hour = Wire.read();
    rval = Wire.read();		// week dummy read
    r_day = Wire.read();
    r_mon = Wire.read();
    r_year = Wire.read();
}
